@@ -23,10 +23,11 @@ from stevedore import enabled
 from blazar.db import api as db_api
 from blazar.db import exceptions as db_ex
 from blazar import exceptions as common_ex
-from blazar.i18n import _
 from blazar import manager
 from blazar.manager import exceptions
+from blazar import monitor
 from blazar.notification import api as notification_api
+from blazar import status
 from blazar.utils import service as service_utils
 from blazar.utils import trusts
 
@@ -39,7 +40,11 @@ manager_opts = [
                default=60,
                help='Minutes prior to the end of a lease in which actions '
                     'like notification and snapshot are taken. If this is '
-                    'set to 0, then these actions are not taken.')
+                    'set to 0, then these actions are not taken.'),
+    cfg.IntOpt('event_max_retries',
+               default=1,
+               max=50,
+               help='Number of times to retry an event action.')
 ]
 
 CONF = cfg.CONF
@@ -48,6 +53,8 @@ LOG = logging.getLogger(__name__)
 
 LEASE_DATE_FORMAT = "%Y-%m-%d %H:%M"
 
+EVENT_INTERVAL = 10
+
 
 class ManagerService(service_utils.RPCServer):
     """Service class for the blazar-manager service.
@@ -61,10 +68,13 @@ class ManagerService(service_utils.RPCServer):
         super(ManagerService, self).__init__(target)
         self.plugins = self._get_plugins()
         self.resource_actions = self._setup_actions()
+        self.monitors = monitor.load_monitors(self.plugins)
 
     def start(self):
         super(ManagerService, self).start()
-        self.tg.add_timer(10, self._event)
+        self.tg.add_timer(EVENT_INTERVAL, self._event)
+        for m in self.monitors:
+            m.start_monitoring()
 
     def _get_plugins(self):
         """Return dict of resource-plugin class pairs."""
@@ -130,33 +140,60 @@ class ManagerService(service_utils.RPCServer):
         status to 'DONE'.
         """
         LOG.debug('Trying to get event from DB.')
-        event = db_api.event_get_first_sorted_by_filters(
+        events = db_api.event_get_all_sorted_by_filters(
             sort_key='time',
             sort_dir='asc',
-            filters={'status': 'UNDONE'}
+            filters={'status': status.event.UNDONE,
+                     'time': {'op': 'le',
+                              'border': datetime.datetime.utcnow()}}
         )
 
-        if not event:
+        if not events:
             return
 
-        if event['time'] < datetime.datetime.utcnow():
-            db_api.event_update(event['id'], {'status': 'IN_PROGRESS'})
-            event_type = event['event_type']
-            event_fn = getattr(self, event_type, None)
-            if event_fn is None:
-                raise exceptions.EventError(error='Event type %s is not '
-                                                  'supported' % event_type)
+        LOG.info("Trying to execute events: %s", events)
+        for event in events:
+            db_api.event_update(event['id'],
+                                {'status': status.event.IN_PROGRESS})
             try:
-                eventlet.spawn_n(service_utils.with_empty_context(event_fn),
-                                 event['lease_id'], event['id'])
-                lease = db_api.lease_get(event['lease_id'])
-                with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
-                    self._send_notification(lease,
-                                            ctx,
-                                            events=['event.%s' % event_type])
+                eventlet.spawn_n(
+                    service_utils.with_empty_context(self._exec_event),
+                    event)
             except Exception:
-                db_api.event_update(event['id'], {'status': 'ERROR'})
-                LOG.exception(_('Error occurred while event handling.'))
+                db_api.event_update(event['id'],
+                                    {'status': status.event.ERROR})
+                LOG.exception('Error occurred while event %s handling.',
+                              event['id'])
+
+    def _exec_event(self, event):
+        """Execute an event function"""
+        event_fn = getattr(self, event['event_type'], None)
+        if event_fn is None:
+            raise exceptions.EventError(
+                error='Event type %s is not supported'
+                      % event['event_type'])
+        try:
+            event_fn(lease_id=event['lease_id'], event_id=event['id'])
+        except common_ex.InvalidStatus:
+            now = datetime.datetime.utcnow()
+            if now < event['time'] + datetime.timedelta(
+                    seconds=CONF.manager.event_max_retries * 10):
+                # Set the event status UNDONE for retrying the event
+                db_api.event_update(event['id'],
+                                    {'status': status.event.UNDONE})
+            else:
+                db_api.event_update(event['id'],
+                                    {'status': status.event.ERROR})
+                LOG.exception('Error occurred while handling event.')
+        except Exception:
+            db_api.event_update(event['id'],
+                                {'status': status.event.ERROR})
+            LOG.exception('Error occurred while handling event.')
+        else:
+            lease = db_api.lease_get(event['lease_id'])
+            with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
+                self._send_notification(
+                    lease, ctx, events=['event.%s' % event['event_type']])
 
     def _date_from_string(self, date_string, date_format=LEASE_DATE_FORMAT):
         try:
@@ -164,9 +201,15 @@ class ManagerService(service_utils.RPCServer):
         except ValueError:
             raise exceptions.InvalidDate(date=date_string,
                                          date_format=date_format)
-
         return date
 
+    def validate_params(self, values, required_params):
+        if isinstance(required_params, list):
+            required_params = set(required_params)
+        missing_attr = required_params - set(values.keys())
+        if missing_attr:
+            raise exceptions.MissingParameter(param=', '.join(missing_attr))
+
     def get_lease(self, lease_id):
         return db_api.lease_get(lease_id)
 
@@ -178,14 +221,20 @@ class ManagerService(service_utils.RPCServer):
 
         Return either the model of created lease or None if any error.
         """
+        lease_values['status'] = status.lease.CREATING
+
         try:
             trust_id = lease_values.pop('trust_id')
         except KeyError:
             raise exceptions.MissingTrustId()
 
+        self.validate_params(lease_values, ['name', 'start_date', 'end_date'])
+
         # Remove and keep event and reservation values
         events = lease_values.pop("events", [])
         reservations = lease_values.pop("reservations", [])
+        for res in reservations:
+            self.validate_params(res, ['resource_type'])
 
         # Create the lease without the reservations
         start_date = lease_values['start_date']
@@ -204,8 +253,12 @@ class ManagerService(service_utils.RPCServer):
         end_date = self._date_from_string(end_date)
 
         if start_date < now:
-            raise common_ex.NotAuthorized(
-                'Start date must later than current date')
+            raise common_ex.InvalidInput(
+                'Start date must be later than current date')
+
+        if end_date <= start_date:
+            raise common_ex.InvalidInput(
+                'End date must be later than start date.')
 
         with trusts.create_ctx_from_trust(trust_id) as ctx:
             # NOTE(priteau): We should not get user_id from ctx, because we are
@@ -217,10 +270,10 @@ class ManagerService(service_utils.RPCServer):
 
             events.append({'event_type': 'start_lease',
                            'time': start_date,
-                           'status': 'UNDONE'})
+                           'status': status.event.UNDONE})
             events.append({'event_type': 'end_lease',
                            'time': end_date,
-                           'status': 'UNDONE'})
+                           'status': status.event.UNDONE})
 
             before_end_date = lease_values.get('before_end_date', None)
             if before_end_date:
@@ -231,7 +284,7 @@ class ManagerService(service_utils.RPCServer):
                     self._check_date_within_lease_limits(before_end_date,
                                                          lease_values)
                 except common_ex.BlazarException as e:
-                    LOG.error("Invalid before_end_date param. %s" % e.message)
+                    LOG.error("Invalid before_end_date param. %s", str(e))
                     raise e
             elif CONF.manager.minutes_before_end_lease > 0:
                 delta = datetime.timedelta(
@@ -240,7 +293,7 @@ class ManagerService(service_utils.RPCServer):
 
             if before_end_date:
                 event = {'event_type': 'before_end_lease',
-                         'status': 'UNDONE'}
+                         'status': status.event.UNDONE}
                 events.append(event)
                 self._update_before_end_event_date(event, before_end_date,
                                                    lease_values)
@@ -284,10 +337,15 @@ class ManagerService(service_utils.RPCServer):
                     raise
 
                 else:
-                    lease = db_api.lease_get(lease['id'])
+                    db_api.lease_update(
+                        lease_id,
+                        {'status': status.lease.PENDING})
+                    lease = db_api.lease_get(lease_id)
                     self._send_notification(lease, ctx, events=['create'])
                     return lease
 
+    @status.lease.lease_status(
+        transition=status.lease.UPDATING, result_in=status.lease.STABLE)
     def update_lease(self, lease_id, values):
         if not values:
             return db_api.lease_get(lease_id)
@@ -325,21 +383,21 @@ class ManagerService(service_utils.RPCServer):
 
         if (lease['start_date'] < now and
                 values['start_date'] != lease['start_date']):
-            raise common_ex.NotAuthorized(
+            raise common_ex.InvalidInput(
                 'Cannot modify the start date of already started leases')
 
         if (lease['start_date'] > now and
                 values['start_date'] < now):
-            raise common_ex.NotAuthorized(
-                'Start date must later than current date')
+            raise common_ex.InvalidInput(
+                'Start date must be later than current date')
 
         if lease['end_date'] < now:
-            raise common_ex.NotAuthorized(
+            raise common_ex.InvalidInput(
                 'Terminated leases can only be renamed')
 
         if (values['end_date'] < now or
            values['end_date'] < values['start_date']):
-            raise common_ex.NotAuthorized(
+            raise common_ex.InvalidInput(
                 'End date must be later than current and start date')
 
         with trusts.create_ctx_from_trust(lease['trust_id']):
@@ -349,7 +407,7 @@ class ManagerService(service_utils.RPCServer):
                     self._check_date_within_lease_limits(before_end_date,
                                                          values)
                 except common_ex.BlazarException as e:
-                    LOG.error("Invalid before_end_date param. %s" % e.message)
+                    LOG.error("Invalid before_end_date param. %s", str(e))
                     raise e
 
             # TODO(frossigneux) rollback if an exception is raised
@@ -360,8 +418,8 @@ class ManagerService(service_utils.RPCServer):
                 v['start_date'] = values['start_date']
                 v['end_date'] = values['end_date']
                 try:
-                    v.update(filter(lambda x: x['id'] == reservation['id'],
-                                    reservations).pop())
+                    v.update([r for r in reservations
+                              if r['id'] == reservation['id']].pop())
                 except KeyError:
                     raise exceptions.MissingParameter(param='reservation ID')
                 except IndexError:
@@ -416,6 +474,8 @@ class ManagerService(service_utils.RPCServer):
 
         return lease
 
+    @status.lease.lease_status(transition=status.lease.DELETING,
+                               result_in=(status.lease.ERROR,))
     def delete_lease(self, lease_id):
         lease = self.get_lease(lease_id)
         if (datetime.datetime.utcnow() >= lease['start_date'] and
@@ -425,28 +485,30 @@ class ManagerService(service_utils.RPCServer):
                 'asc',
                 {
                     'lease_id': lease_id,
-                    'event_type': 'start_lease',
-                    'status': 'DONE'
+                    'event_type': 'start_lease'
                 }
             )
             if not start_event:
-                raise common_ex.BlazarException('Invalid event status')
+                raise common_ex.BlazarException(
+                    'start_lease event for lease %s not found' % lease_id)
             end_event = db_api.event_get_first_sorted_by_filters(
                 'lease_id',
                 'asc',
                 {
                     'lease_id': lease_id,
                     'event_type': 'end_lease',
-                    'status': 'UNDONE'
+                    'status': status.event.UNDONE
                 }
             )
             if not end_event:
-                raise common_ex.BlazarException('Invalid event status')
-            db_api.event_update(end_event['id'], {'status': 'IN_PROGRESS'})
+                raise common_ex.BlazarException(
+                    'end_lease event for lease %s not found' % lease_id)
+            db_api.event_update(end_event['id'],
+                                {'status': status.event.IN_PROGRESS})
 
         with trusts.create_ctx_from_trust(lease['trust_id']) as ctx:
             for reservation in lease['reservations']:
-                if reservation['status'] != 'deleted':
+                if reservation['status'] != status.reservation.DELETED:
                     plugin = self.plugins[reservation['resource_type']]
                     try:
                         plugin.on_end(reservation['resource_id'])
@@ -457,18 +519,23 @@ class ManagerService(service_utils.RPCServer):
             db_api.lease_destroy(lease_id)
             self._send_notification(lease, ctx, events=['delete'])
 
+    @status.lease.lease_status(
+        transition=status.lease.STARTING,
+        result_in=(status.lease.ACTIVE, status.lease.ERROR))
     def start_lease(self, lease_id, event_id):
         lease = self.get_lease(lease_id)
         with trusts.create_ctx_from_trust(lease['trust_id']):
-            self._basic_action(lease_id, event_id, 'on_start', 'active')
+            self._basic_action(lease_id, event_id, 'on_start',
+                               status.reservation.ACTIVE)
 
+    @status.lease.lease_status(
+        transition=status.lease.TERMINATING,
+        result_in=(status.lease.TERMINATED, status.lease.ERROR))
     def end_lease(self, lease_id, event_id):
         lease = self.get_lease(lease_id)
-        for reservation in lease['reservations']:
-            db_api.reservation_update(reservation['id'],
-                                      {'status': 'completed'})
         with trusts.create_ctx_from_trust(lease['trust_id']):
-            self._basic_action(lease_id, event_id, 'on_end', 'deleted')
+            self._basic_action(lease_id, event_id, 'on_end',
+                               status.reservation.DELETED)
 
     def before_end_lease(self, lease_id, event_id):
         lease = self.get_lease(lease_id)
@@ -480,23 +547,26 @@ class ManagerService(service_utils.RPCServer):
         """Commits basic lease actions such as starting and ending."""
         lease = self.get_lease(lease_id)
 
-        event_status = 'DONE'
+        event_status = status.event.DONE
         for reservation in lease['reservations']:
             resource_type = reservation['resource_type']
             try:
+                if reservation_status is not None:
+                    if not status.reservation.is_valid_transition(
+                            reservation['status'], reservation_status):
+                        raise common_ex.InvalidStatus
                 self.resource_actions[resource_type][action_time](
                     reservation['resource_id']
                 )
             except common_ex.BlazarException:
                 LOG.exception("Failed to execute action %(action)s "
-                              "for lease %(lease)s"
-                              % {
-                                  'action': action_time,
-                                  'lease': lease_id,
-                              })
-                event_status = 'ERROR'
-                db_api.reservation_update(reservation['id'],
-                                          {'status': 'error'})
+                              "for lease %(lease)s",
+                              {'action': action_time,
+                               'lease': lease_id})
+                event_status = status.event.ERROR
+                db_api.reservation_update(
+                    reservation['id'],
+                    {'status': status.reservation.ERROR})
             else:
                 if reservation_status is not None:
                     db_api.reservation_update(reservation['id'],
@@ -504,6 +574,8 @@ class ManagerService(service_utils.RPCServer):
 
         db_api.event_update(event_id, {'status': event_status})
 
+        return event_status
+
     def _create_reservation(self, values):
         resource_type = values['resource_type']
         if resource_type not in self.plugins:
@@ -511,7 +583,7 @@ class ManagerService(service_utils.RPCServer):
         reservation_values = {
             'lease_id': values['lease_id'],
             'resource_type': resource_type,
-            'status': 'pending'
+            'status': status.reservation.PENDING
         }
         reservation = db_api.reservation_create(reservation_values)
         resource_id = self.plugins[resource_type].reserve_resource(
@@ -537,9 +609,10 @@ class ManagerService(service_utils.RPCServer):
         event['time'] = before_end_date
         if event['time'] < lease['start_date']:
             LOG.warning("Start_date greater than before_end_date. "
-                        "Setting before_end_date to %s for lease %s"
-                        % (lease['start_date'], lease.get('id',
-                           lease.get('name'))))
+                        "Setting before_end_date to %(start_date)s for "
+                        "lease %(id_name)s",
+                        {'start_date': lease['start_date'],
+                         'id_name': lease.get('id', lease.get('name'))})
             event['time'] = lease['start_date']
 
     def _update_before_end_event(self, old_lease, new_lease,
@@ -564,8 +637,8 @@ class ManagerService(service_utils.RPCServer):
 
             self._update_before_end_event_date(update_values, before_end_date,
                                                new_lease)
-            if event['status'] == 'DONE':
-                update_values['status'] = 'UNDONE'
+            if event['status'] == status.event.DONE:
+                update_values['status'] = status.event.UNDONE
                 notifications.append('event.before_end_lease.stop')
 
             db_api.event_update(event['id'], update_values)
