@@ -22,6 +22,9 @@ class DummyVMPlugin(base.BasePlugin):
     title = 'Dummy VM Plugin'
     description = 'This plugin does nothing.'
 
+    def reserve_resource(self, reservation_id, values):
+        return None
+
     def on_start(self, resource_id):
         """Dummy VM plugin does nothing."""
         return 'VM %s should be waked up this moment.' % resource_id
